@@ -1,7 +1,7 @@
 /*
- * Copyright 2022-2023 Soren Stoutner <soren@stoutner.com>.
+ * Copyright 2022-2024 Soren Stoutner <soren@stoutner.com>.
  *
- * This file is part of Privacy Browser PC <https://www.stoutner.com/privacy-browser-pc>.
+ * This file is part of Privacy Browser PC <https://www.stoutner.com/privacy-browser-pc/>.
  *
  * Privacy Browser PC is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +17,12 @@
  * along with Privacy Browser PC.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef URLREQUESTINTERCEPTOR_H
-#define URLREQUESTINTERCEPTOR_H
+#ifndef URL_REQUEST_INTERCEPTOR_H
+#define URL_REQUEST_INTERCEPTOR_H
+
+// Application headers.
+#include "structs/RequestStruct.h"
+#include "widgets/PrivacyWebEngineView.h"
 
 // Qt framework headers.
 #include <QtWebEngineCore>
@@ -30,7 +34,7 @@ class UrlRequestInterceptor : public QWebEngineUrlRequestInterceptor
 
 public:
     // The default constructor.
-    UrlRequestInterceptor(QObject *parentObjectPointer = 0);
+    UrlRequestInterceptor(PrivacyWebEngineView *privacyWebEngineViewPointer);
 
     // The public functions.
     void interceptRequest(QWebEngineUrlRequestInfo &urlRequestInfo) override;
@@ -39,5 +43,11 @@ signals:
     // The signals.
     void applyDomainSettings(const QString &hostname) const;
     void displayHttpPingDialog(const QString &httpPingUrl) const;
+    void newMainFrameResource() const;
+    void requestProcessed(RequestStruct *requestStruct) const;
+
+private:
+    // The private variables.
+    PrivacyWebEngineView *privacyWebEngineViewPointer;
 };
 #endif
