@@ -1,7 +1,7 @@
 /*
- * Copyright 2022-2023 Soren Stoutner <soren@stoutner.com>.
+ * Copyright 2022-2024 Soren Stoutner <soren@stoutner.com>.
  *
- * This file is part of Privacy Browser PC <https://www.stoutner.com/privacy-browser-pc>.
+ * This file is part of Privacy Browser PC <https://www.stoutner.com/privacy-browser-pc/>.
  *
  * Privacy Browser PC is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,9 +18,11 @@
  */
 
 // Application headers.
+#include "GlobalVariables.h"
 #include "databases/BookmarksDatabase.h"
 #include "databases/CookiesDatabase.h"
 #include "databases/DomainsDatabase.h"
+#include "helpers/FilterListHelper.h"
 #include "windows/BrowserWindow.h"
 
 // KDE Frameworks headers.
@@ -34,6 +36,9 @@
 #include <QCommandLineParser>
 #include <QFile>
 
+// Declare the global variables.
+FilterListHelper *globalFilterListHelperPointer;
+
 int main(int argc, char *argv[])
 {
     // Create the application.
@@ -46,7 +51,7 @@ int main(int argc, char *argv[])
     KCrash::initialize();
 
     // Instantiate about data, setting the component name, the display name, and the version.
-    KAboutData aboutData(QStringLiteral("privacybrowser"), i18nc("Program Name", "Privacy Browser"), QStringLiteral("0.5"));
+    KAboutData aboutData(QStringLiteral("privacybrowser"), i18nc("Program Name", "Privacy Browser"), QStringLiteral("0.6"));
 
     // Add the author name, job description, email address, and website.
     aboutData.addAuthor(i18nc("Developer Information", "Soren Stoutner"),i18nc("Developer Information", "Principal developer"), QStringLiteral("soren@stoutner.com"),
@@ -54,7 +59,7 @@ int main(int argc, char *argv[])
 
     // Populate additional about data info.
     aboutData.setBugAddress("https://redmine.stoutner.com/projects/privacy-browser-pc/issues");
-    aboutData.setCopyrightStatement(i18nc("Copyright", "Copyright 2016-2017,2021-2023 Soren Stoutner <soren@stoutner.com>"));
+    aboutData.setCopyrightStatement(i18nc("Copyright", "Copyright 2016-2017,2021-2024 Soren Stoutner <soren@stoutner.com>"));
     aboutData.setDesktopFileName(QStringLiteral("com.stoutner.privacybrowser"));
     aboutData.setHomepage(QStringLiteral("https://www.stoutner.com/privacy-browser-pc/"));
     //aboutData.setLicense(KAboutLicense::GPL_V3, KAboutLicense::OrLaterVersions);  <https://redmine.stoutner.com/issues/822>
@@ -92,6 +97,9 @@ int main(int argc, char *argv[])
     CookiesDatabase::addDatabase();
     DomainsDatabase::addDatabase();
 
+    // Populate the filter lists.
+    globalFilterListHelperPointer = new FilterListHelper;
+
     // Create the main window.
     BrowserWindow *browserWindowPointer = new BrowserWindow();
 
