@@ -1,7 +1,7 @@
 /*
- * Copyright 2022-2023 Soren Stoutner <soren@stoutner.com>.
+ * Copyright 2022-2024 Soren Stoutner <soren@stoutner.com>.
  *
- * This file is part of Privacy Browser PC <https://www.stoutner.com/privacy-browser-pc>.
+ * This file is part of Privacy Browser PC <https://www.stoutner.com/privacy-browser-pc/>.
  *
  * Privacy Browser PC is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,8 +20,11 @@
 // Application headers.
 #include "PrivacyWebEngineView.h"
 #include "Settings.h"
+#include "ui_HttpAuthenticationDialog.h"
 #include "databases/CookiesDatabase.h"
 #include "databases/DomainsDatabase.h"
+#include "dialogs/HttpAuthenticationDialog.h"
+#include "helpers/FilterListHelper.h"
 #include "interceptors/UrlRequestInterceptor.h"
 #include "windows/BrowserWindow.h"
 
@@ -30,7 +33,7 @@
 #include <QMenu>
 
 // Construct the class.
-PrivacyWebEngineView::PrivacyWebEngineView() : QWebEngineView(nullptr)
+PrivacyWebEngineView::PrivacyWebEngineView(QWidget *parentWidgetPointer) : QWebEngineView(parentWidgetPointer)
 {
     // Create an off-the-record profile (the default when no profile name is specified).
     webEngineProfilePointer = new QWebEngineProfile(QLatin1String(""));
@@ -45,16 +48,19 @@ PrivacyWebEngineView::PrivacyWebEngineView() : QWebEngineView(nullptr)
     webEngineSettingsPointer = webEnginePagePointer->settings();
 
     // Instantiate the URL request interceptor.
-    UrlRequestInterceptor *urlRequestInterceptorPointer = new UrlRequestInterceptor();
+    UrlRequestInterceptor *urlRequestInterceptorPointer = new UrlRequestInterceptor(this);
 
     // Set the URL request interceptor.
     webEngineProfilePointer->setUrlRequestInterceptor(urlRequestInterceptorPointer);
 
-    // Reapply the domain settings when the host changes.
+    // Connect the URL request interceptor signals.
     connect(urlRequestInterceptorPointer, SIGNAL(applyDomainSettings(const QString&)), this, SLOT(applyDomainSettingsWithoutReloading(const QString&)));
-
-    // Display HTTP Ping blocked dialogs.
+    connect(urlRequestInterceptorPointer, SIGNAL(newMainFrameResource()), this, SLOT(clearRequestsList()));
     connect(urlRequestInterceptorPointer, SIGNAL(displayHttpPingDialog(const QString&)), this, SLOT(displayHttpPingDialog(const QString&)));
+    connect(urlRequestInterceptorPointer, SIGNAL(requestProcessed(RequestStruct*)), this, SLOT(storeRequest(RequestStruct*)));
+
+    // Handle HTTP authentication requests.
+    connect(webEnginePagePointer, SIGNAL(authenticationRequired(const QUrl&, QAuthenticator*)), this, SLOT(handleAuthenticationRequest(const QUrl&, QAuthenticator*)));
 }
 
 void PrivacyWebEngineView::addCookieToList(const QNetworkCookie &cookie) const
@@ -69,7 +75,7 @@ void PrivacyWebEngineView::addCookieToList(const QNetworkCookie &cookie) const
         CookiesDatabase::updateCookie(cookie);
 
     // Update the cookies action.
-    emit updateCookiesAction(cookieListPointer->size());
+    emit numberOfCookiesChanged(cookieListPointer->size());
 }
 
 void PrivacyWebEngineView::applyDomainSettingsWithoutReloading(const QString &hostname)
@@ -92,90 +98,70 @@ void PrivacyWebEngineView::applyDomainSettings(const QString &hostname, const bo
         // Set the JavaScript status.
         switch (domainQuery.value(DomainsDatabase::JAVASCRIPT).toInt())
         {
-            // Set the default JavaScript status.
-            case (DomainsDatabase::SYSTEM_DEFAULT):
-            {
-                webEngineSettingsPointer->setAttribute(QWebEngineSettings::JavascriptEnabled, Settings::javaScriptEnabled());
-
-                break;
-            }
-
-            // Enable JavaScript.
-            case (DomainsDatabase::ENABLED):
-            {
-                webEngineSettingsPointer->setAttribute(QWebEngineSettings::JavascriptEnabled, true);
-
-                break;
-            }
-
-            // Disable JavaScript.
-            case (DomainsDatabase::DISABLED):
-            {
-                webEngineSettingsPointer->setAttribute(QWebEngineSettings::JavascriptEnabled, false);
-
-                break;
-            }
+            case (DomainsDatabase::SYSTEM_DEFAULT): webEngineSettingsPointer->setAttribute(QWebEngineSettings::JavascriptEnabled, Settings::javaScriptEnabled()); break;
+            case (DomainsDatabase::ENABLED): webEngineSettingsPointer->setAttribute(QWebEngineSettings::JavascriptEnabled, true); break;
+            case (DomainsDatabase::DISABLED): webEngineSettingsPointer->setAttribute(QWebEngineSettings::JavascriptEnabled, false); break;
         }
 
         // Set the local storage status.
         switch (domainQuery.value(DomainsDatabase::LOCAL_STORAGE).toInt())
         {
-            // Set the default local storage status.
-            case (DomainsDatabase::SYSTEM_DEFAULT):
-            {
-                localStorageEnabled = Settings::localStorageEnabled();
-
-                break;
-            }
-
-            // Enable local storage.
-            case (DomainsDatabase::ENABLED):
-            {
-                localStorageEnabled = true;
-
-                break;
-            }
-
-            // Disable local storage.
-            case (DomainsDatabase::DISABLED):
-            {
-                localStorageEnabled = false;
-
-                break;
-            }
+            case (DomainsDatabase::SYSTEM_DEFAULT): localStorageEnabled = Settings::localStorageEnabled(); break;
+            case (DomainsDatabase::ENABLED): localStorageEnabled = true; break;
+            case (DomainsDatabase::DISABLED): localStorageEnabled = false; break;
         }
 
-        // Set the DOM storage status.
+        // Set the DOM storage status.  QWebEngineSettings confusingly calls this local storage.
         switch (domainQuery.value(DomainsDatabase::DOM_STORAGE).toInt())
         {
-            // Set the default DOM storage status.  QWebEngineSettings confusingly calls this local storage.
-            case (DomainsDatabase::SYSTEM_DEFAULT):
-            {
-                webEngineSettingsPointer->setAttribute(QWebEngineSettings::LocalStorageEnabled, Settings::domStorageEnabled());
-
-                break;
-            }
-
-            // Enable DOM storage.  QWebEngineSettings confusingly calls this local storage.
-            case (DomainsDatabase::ENABLED):
-            {
-                webEngineSettingsPointer->setAttribute(QWebEngineSettings::LocalStorageEnabled, true);
-
-                break;
-            }
-
-            // Disable DOM storage.  QWebEngineSettings confusingly calls this local storage.
-            case (DomainsDatabase::DISABLED):
-            {
-                webEngineSettingsPointer->setAttribute(QWebEngineSettings::LocalStorageEnabled, false);
-
-                break;
-            }
+            case (DomainsDatabase::SYSTEM_DEFAULT): webEngineSettingsPointer->setAttribute(QWebEngineSettings::LocalStorageEnabled, Settings::domStorageEnabled()); break;
+            case (DomainsDatabase::ENABLED): webEngineSettingsPointer->setAttribute(QWebEngineSettings::LocalStorageEnabled, true); break;
+            case (DomainsDatabase::DISABLED): webEngineSettingsPointer->setAttribute(QWebEngineSettings::LocalStorageEnabled, false); break;
         }
 
         // Set the user agent.
         webEngineProfilePointer->setHttpUserAgent(UserAgentHelper::getResultingDomainSettingsUserAgent(domainQuery.value(DomainsDatabase::USER_AGENT).toString()));
 
+        // Set the UltraPrivacy status.
+        switch (domainQuery.value(DomainsDatabase::ULTRAPRIVACY).toInt())
+        {
+            case (DomainsDatabase::SYSTEM_DEFAULT): ultraPrivacyEnabled = Settings::ultraPrivacyEnabled(); break;
+            case (DomainsDatabase::ENABLED): ultraPrivacyEnabled = true; break;
+            case (DomainsDatabase::DISABLED): ultraPrivacyEnabled = false; break;
+        }
+
+        // Set the UltraList status.
+        switch (domainQuery.value(DomainsDatabase::ULTRALIST).toInt())
+        {
+            case (DomainsDatabase::SYSTEM_DEFAULT): ultraListEnabled = Settings::ultraListEnabled(); break;
+            case (DomainsDatabase::ENABLED): ultraListEnabled = true; break;
+            case (DomainsDatabase::DISABLED): ultraListEnabled = false; break;
+        }
+
+        // Set the EasyPrivacy status.
+        switch (domainQuery.value(DomainsDatabase::EASYPRIVACY).toInt())
+        {
+            case (DomainsDatabase::SYSTEM_DEFAULT): easyPrivacyEnabled = Settings::easyPrivacyEnabled(); break;
+            case (DomainsDatabase::ENABLED): easyPrivacyEnabled = true; break;
+            case (DomainsDatabase::DISABLED): easyPrivacyEnabled = false; break;
+        }
+
+        // Set the EasyList status.
+        switch (domainQuery.value(DomainsDatabase::EASYLIST).toInt())
+        {
+            case (DomainsDatabase::SYSTEM_DEFAULT): easyListEnabled = Settings::easyListEnabled(); break;
+            case (DomainsDatabase::ENABLED): easyListEnabled = true; break;
+            case (DomainsDatabase::DISABLED): easyListEnabled = false; break;
+        }
+
+        // Set the Fanboy's Annoyance List status.
+        switch (domainQuery.value(DomainsDatabase::FANBOYS_ANNOYANCE_LIST).toInt())
+        {
+            case (DomainsDatabase::SYSTEM_DEFAULT): fanboysAnnoyanceListEnabled = Settings::fanboysAnnoyanceListEnabled(); break;
+            case (DomainsDatabase::ENABLED): fanboysAnnoyanceListEnabled = true; break;
+            case (DomainsDatabase::DISABLED): fanboysAnnoyanceListEnabled = false; break;
+        }
+
         // Check if a custom zoom factor is set.
         if (domainQuery.value(DomainsDatabase::ZOOM_FACTOR).toInt())
         {
@@ -205,6 +191,13 @@ void PrivacyWebEngineView::applyDomainSettings(const QString &hostname, const bo
         // Set the user agent.
         webEngineProfilePointer->setHttpUserAgent(UserAgentHelper::getUserAgentFromDatabaseName(Settings::userAgent()));
 
+        // Set the status for each filter list.
+        ultraPrivacyEnabled = Settings::ultraPrivacyEnabled();
+        ultraListEnabled = Settings::ultraListEnabled();
+        easyPrivacyEnabled = Settings::easyPrivacyEnabled();
+        easyListEnabled = Settings::easyListEnabled();
+        fanboysAnnoyanceListEnabled = Settings::fanboysAnnoyanceListEnabled();
+
         // Store the zoom factor.
         defaultZoomFactor = Settings::zoomFactor();
     }
@@ -216,10 +209,30 @@ void PrivacyWebEngineView::applyDomainSettings(const QString &hostname, const bo
     if (reloadWebsite)
         reload();
 
+    // Reset the HTTP authentication dialog counter.
+    httpAuthenticationDialogsDisplayed = 0;
+
     // Update the UI.
     emit updateUi(this);
 }
 
+void PrivacyWebEngineView::clearRequestsList()
+{
+    // Reset the number of blocked requests.
+    blockedRequestsVector[ULTRAPRIVACY] = 0;
+    blockedRequestsVector[ULTRALIST] = 0;
+    blockedRequestsVector[EASYPRIVACY] = 0;
+    blockedRequestsVector[EASYLIST] = 0;
+    blockedRequestsVector[FANBOYS_ANNOYANCE_LIST] = 0;
+    blockedRequestsVector[TOTAL] = 0;
+
+    // Clear the requests list.
+    requestsListPointer->clear();
+
+    // Update the blocked requests action.
+    emit(requestBlocked(blockedRequestsVector));
+}
+
 void PrivacyWebEngineView::contextMenuEvent(QContextMenuEvent *contextMenuEvent) {
     // Get a handle for the
     QWebEnginePage *webEnginePagePointer = page();
@@ -259,9 +272,9 @@ QWebEngineView* PrivacyWebEngineView::createWindow(QWebEnginePage::WebWindowType
     {
         case QWebEnginePage::WebBrowserTab:
         {
-            // Create the new tab and return the privacy WebEngine view pointer.  `true` removes the focus from the blank URL line edit.
+            // Create the new tab and return the privacy WebEngine view pointer.  `true` removes the focus from the blank URL line edit.  `true` adds the new tab adjacent to the current tab.
             // The new privacy WebEngine view pointer is returned so it can be populated with the link from the context menu.
-            return browserWindowPointer->tabWidgetPointer->addTab(true);
+            return browserWindowPointer->tabWidgetPointer->addTab(true, true);
         }
 
         case QWebEnginePage::WebBrowserWindow:
@@ -278,9 +291,10 @@ QWebEngineView* PrivacyWebEngineView::createWindow(QWebEnginePage::WebWindowType
 
         case QWebEnginePage::WebBrowserBackgroundTab:
         {
-            // Create the new tab and return the privacy WebEngine view pointer.  `false` does not clear the URL line edit.  `true` creates a background tab.
+            // Create the new tab and return the privacy WebEngine view pointer.  `false` does not clear the URL line edit.  `true` adds the new tab adjacent to the current tab.
+            // `true` creates a background tab.
             // The new privacy WebEngine view pointer is returned so it can be populated with the link from the context menu.
-            return browserWindowPointer->tabWidgetPointer->addTab(false, true);
+            return browserWindowPointer->tabWidgetPointer->addTab(false, true, true);
         }
 
         default:
@@ -297,6 +311,21 @@ void PrivacyWebEngineView::displayHttpPingDialog(const QString &httpPingUrl) con
     emit displayHttpPingBlockedDialog(httpPingUrl);
 }
 
+void PrivacyWebEngineView::handleAuthenticationRequest(const QUrl &requestUrl, QAuthenticator *authenticatorPointer)
+{
+    // Only display the HTTP authentication dialog if it hasn't already been displayed three times for this URL.
+    if (httpAuthenticationDialogsDisplayed < 3) {
+        // Increment the HTTP authentication dialog display counter.
+        ++httpAuthenticationDialogsDisplayed;
+
+        // Instantiate an HTTP authentication dialog.
+        HttpAuthenticationDialog *httpAuthenticationDialogPointer = new HttpAuthenticationDialog(parentWidget(), requestUrl, authenticatorPointer);
+
+        // Display the dialog.  This must be `exec()` instead of `show()` so that the website doesn't proceed before populating the authentication pointer.
+        httpAuthenticationDialogPointer->exec();
+    }
+}
+
 void PrivacyWebEngineView::removeCookieFromList(const QNetworkCookie &cookie) const
 {
     //qDebug() << "Remove cookie:  " << cookie.toRawForm();
@@ -305,5 +334,33 @@ void PrivacyWebEngineView::removeCookieFromList(const QNetworkCookie &cookie) co
     cookieListPointer->remove(cookie);
 
     // Update the cookies action.
-    emit updateCookiesAction(cookieListPointer->size());
+    emit numberOfCookiesChanged(cookieListPointer->size());
+}
+
+void PrivacyWebEngineView::storeRequest(RequestStruct *requestStructPointer)
+{
+    // Store the request struct in the list.
+    requestsListPointer->append(requestStructPointer);
+
+    // Track blocked requests.
+    if (requestStructPointer->dispositionInt == FilterListHelper::BLOCKED)
+    {
+        // Update the individual filter list block counters.
+        if (requestStructPointer->filterListTitle == QLatin1String("UltraPrivacy"))
+            ++blockedRequestsVector[ULTRAPRIVACY];
+        else if (requestStructPointer->filterListTitle == QLatin1String("UltraList"))
+            ++blockedRequestsVector[ULTRALIST];
+        else if (requestStructPointer->filterListTitle == QLatin1String("EasyPrivacy"))
+            ++blockedRequestsVector[EASYPRIVACY];
+        else if (requestStructPointer->filterListTitle == QLatin1String("EasyList"))
+            ++blockedRequestsVector[EASYLIST];
+        else if (requestStructPointer->filterListTitle == QLatin1String("Fanboy's Annoyance List"))
+            ++blockedRequestsVector[FANBOYS_ANNOYANCE_LIST];
+
+        // Update the total requests blocked counter.
+        ++blockedRequestsVector[TOTAL];
+
+        // Update the blocked requests action.
+        emit(requestBlocked(blockedRequestsVector));
+    }
 }
